#include <windows.h>
#include <fcntl.h>
#include <io.h>
#include <lzexpand.h>
#include <string.h>
#include "global.h"


//****************************************************************************
// Function: InstallRasterVector
//
// Purpose: Installs a raster or vector font to the system
//
// Parameters:
//    lpszFile     == Name of file to install including path
//    lpszTitle    == Name of file without path
//
// Returns: Boolean value indicating success or failure
//
// Comments:
//
// History:  Date       Author        Reason
//           3/4/95     GGB           Created
//****************************************************************************

BOOL InstallRasterVector(LPSTR lpszFile, LPSTR lpszTitle)
   {           
   char lpszDesc[256];
   LPSTR lpszName;
   char szTemp[256];
   char szDst[256];
   char szName[256];
   OFSTRUCT ofStrSrc;
   OFSTRUCT ofStrDest;
   HFILE hfSrcFile, hfDstFile;
   BOOL  bRaster = TRUE;
   TEXTMETRIC tm;
   HDC hdcScreen;
   HFONT hOldFont;
   LOGFONT lf;
   char szBuf[256];
   
   szBuf[0] = '\0';
   szTemp[0] = '\0';
   szDst[0] = '\0';
   szName[0] = '\0';
   lpszDesc[0] = '\0';
   AnsiUpperBuff(lpszTitle,lstrlen(lpszTitle));
   // Read the description table of the .FON font file
   if (!(ReadExeDesc(lpszFile,lpszDesc,&lf)))
      {
      MessageBox(GetFocus(),"Not a valid font file","fonts",MB_OK);
      return FALSE;
      }
   // Is it a vector font?
   if (_fstrstr(lpszDesc,"CONTINUOUSSCALING"))
     bRaster = FALSE;
   if (!(lpszName = _fstrstr(lpszDesc,":")))
     return FALSE;
   lstrcpy(szName,lpszName+2);
   if (!bRaster) 
      lstrcat(szName," (Plotter)");
   // Is this font already installed?
   if (GetProfileString("fonts", szName, NULL, szTemp, 256))
        {
        MessageBox(GetFocus(),"Font already installed","fonts",MB_OK);
        return FALSE;
        }
   hfSrcFile = LZOpenFile(lpszFile, &ofStrSrc, OF_READ);
   GetWindowsDirectory(szDst,256);
   AnsiUpperBuff(szDst,lstrlen(szDst));
   lstrcat(szDst,"\\SYSTEM\\");
   lstrcat(szDst,lpszTitle);
   /* Create the destination file. */
   hfDstFile = LZOpenFile(szDst, &ofStrDest, OF_CREATE);
   // Make sure it isn't already in the system directory
   if (lstrcmp(szDst,lpszFile) == 0)
      {
      /* Close the files. */
      LZClose(hfSrcFile);
      LZClose(hfDstFile);
      MessageBox(GetFocus(),"Font already in the 'System' directory","fonts",MB_OK);
      return FALSE;
      }
   /* Copy the source file to the destination file. */
   LZCopy(hfSrcFile, hfDstFile);
   /* Close the files. */
   LZClose(hfSrcFile);
   LZClose(hfDstFile);
   // Add it to the system
   AddFontResource(szDst);
   // Tell everyone else what you have done
   SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
   WriteProfileString("fonts",szName,lpszTitle);    
   lstrcpy(szBuf,szName);
   lstrcat(szBuf,"=");
   lstrcat(szBuf,lpszTitle);
   // Add the line to the list box on the main window for informational purposes
   SendMessage(ghWndList, LB_ADDSTRING, 0, (LPARAM)(LPCSTR)szBuf);

   SendMessage(HWND_BROADCAST,WM_WININICHANGE, 0, (LPARAM)(LPCSTR)"fonts"); 
   if (ghCurrFont)
      DeleteObject(ghCurrFont);
   // Now, create a logical font for this font file so we can view it.
   ghCurrFont = CreateFont(0,0,0,0,lf.lfWeight,lf.lfItalic,lf.lfUnderline,lf.lfStrikeOut,
                            lf.lfCharSet,OUT_DEFAULT_PRECIS,
                            CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, lf.lfPitchAndFamily,
                            lf.lfFaceName);
   hdcScreen = GetDC(NULL);
   hOldFont = SelectObject(hdcScreen,ghCurrFont);
   GetTextMetrics(hdcScreen,&tm);
   if (tm.tmPitchAndFamily & TMPF_FIXED_PITCH)
      lstrcpy(ghFontInfo.Pitch,"Variable");
   else
      lstrcpy(ghFontInfo.Pitch,"Fixed");
   SelectObject(hdcScreen,hOldFont);
   ReleaseDC(NULL,hdcScreen);
   lstrcpy(ghFontInfo.FontName,szName);
   if (bRaster)
      lstrcpy(ghFontInfo.FontType,"Raster");
   else
      lstrcpy(ghFontInfo.FontType,"Vector");
   lstrcpy(ghFontInfo.File,szDst);
   lstrcpy(ghFontInfo.FOTFile,"n/a");
   if (bRaster)
      ghFontInfo.Scalable = FALSE;
   else
      ghFontInfo.Scalable = TRUE;
   return TRUE;
   }


//****************************************************************************
// Function: InstallTrueType
//
// Purpose: Installs a TrueType font to the system
//
// Parameters:
//    pszFile     == Name of file to install including path
//    pszTitle    == Name of file without path
//
// Returns: Boolean value indicating success or failure
//
// Comments:
//
// History:  Date       Author        Reason
//           3/4/95     GGB           Created
//****************************************************************************

BOOL InstallTrueType(PSTR pszFile, PSTR pszTitle)
   {           
   LPSTR lpszLongName;
   static char szDst[256];
   static char szFileName[256];
   static char szDstFot[256];
   static char szDstName[256];
   static char szTemp[256];
   OFSTRUCT ofStrSrc;
   OFSTRUCT ofStrDest;
   HFILE hfSrcFile, hfDstFile;
   TEXTMETRIC tm;
   HDC hdcScreen;
   HFONT hOldFont;
   char szBuf[256];
   
   szDst[0] = '\0';
   szBuf[0] = '\0';
   szFileName[0] = '\0';
   szDstFot[0] = '\0';
   szDstName[0] = '\0';
   szTemp[0] = '\0';
   AnsiUpperBuff(pszTitle,lstrlen(pszTitle));
   // Find the facename of the font file
   if (lpszLongName = ReadLongName(pszFile))
     {
     hfSrcFile = LZOpenFile(pszFile, &ofStrSrc, OF_READ);
     GetWindowsDirectory(szDst,256);
     AnsiUpperBuff(szDst,lstrlen(szDst));
     lstrcat(szDst,"\\SYSTEM\\");
     lstrcat(szDst,pszTitle);
     /* Create the destination file. */
     hfDstFile = LZOpenFile(szDst, &ofStrDest, OF_CREATE);
     if (lstrcmp(szDst,pszFile) == 0)
        {
        /* Close the files. */
        LZClose(hfSrcFile);
        LZClose(hfDstFile);
        MessageBox(GetFocus(),"Font already in the 'System' directory","fonts",MB_OK);
        return FALSE;
        }
     /* Copy the source file to the destination file. */
     LZCopy(hfSrcFile, hfDstFile);
     /* Close the files. */
     LZClose(hfSrcFile);
     LZClose(hfDstFile);
     lstrcpy(szDstFot,szDst);
     szDstFot[lstrlen(szDst)-3] = NULL;
     lstrcat(szDstFot,"FOT");
     // Create a .FOT file for the .TTF file
     CreateScalableFontResource(0, szDstFot, szDst, NULL);
     AddFontResource(szDstFot);
     SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
     lstrcpy(szDstName,lpszLongName);
     lstrcpy(szFileName,pszTitle);
     szFileName[lstrlen(szFileName)-3] = NULL;
     lstrcat(szFileName,"FOT");
     lstrcat(szDstName," (TrueType)");
     // Is it already installed?
     if (GetProfileString("fonts", szDstName, NULL, szTemp, 256))
        {
        MessageBox(GetFocus(),"Font already installed","fonts",MB_OK);
        return FALSE;
        }
     WriteProfileString("fonts",szDstName,szFileName);
     lstrcpy(szBuf,szDstName);
     lstrcat(szBuf,"=");
     lstrcat(szBuf,szFileName);
     SendMessage(ghWndList, LB_ADDSTRING, 0, (LPARAM)(LPCSTR)szBuf);
     SendMessage(HWND_BROADCAST,WM_WININICHANGE, 0, (LPARAM)(LPCSTR)"fonts");
     if (ghCurrFont)
        DeleteObject(ghCurrFont);
     // Now, create a logical font for this font file so we can view it.
     ghCurrFont = CreateFont(-60,0,0,0,FW_DONTCARE,0,0,0,DEFAULT_CHARSET,OUT_TT_PRECIS,
                            CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH|FF_DONTCARE,
                            lpszLongName);
     hdcScreen = GetDC(NULL);
     hOldFont = SelectObject(hdcScreen,ghCurrFont);
     GetTextMetrics(hdcScreen,&tm);
     if (tm.tmPitchAndFamily & TMPF_FIXED_PITCH)
        lstrcpy(ghFontInfo.Pitch,"Variable");
     else
        lstrcpy(ghFontInfo.Pitch,"Fixed");
     SelectObject(hdcScreen,hOldFont);
     ReleaseDC(NULL,hdcScreen);
     ghFontInfo.Scalable = TRUE;
     lstrcpy(ghFontInfo.FontName,lpszLongName);
     lstrcpy(ghFontInfo.FontType,"TrueType");
     lstrcpy(ghFontInfo.File,szDst);
     lstrcpy(ghFontInfo.FOTFile,szDstFot);
     return TRUE;
     }
   MessageBox(GetFocus(),"Not a valid TrueType font file","fonts",MB_OK);
   return FALSE;  
   }


//****************************************************************************
// Function: ReadExeDesc
//
// Purpose: Reads the "Description" statement in the .EXE file header.
//
// Parameters:
//    lpszFile     == Name of file to read including path
//    lpszTitle    == "Description" of file to be filled by function
//
// Returns: Boolean value indicating success or failure
//
// Comments:
//
// History:  Date       Author        Reason
//           3/4/95     GGB           Created
//****************************************************************************

BOOL ReadExeDesc(LPSTR lpszFile, LPSTR lpszDesc, LPLOGFONT lf)
   {   
   OFSTRUCT of;
   int      fFile=0;
   OLDEXE OldExe; 
   NEWEXE NewExe;
   long         lNonResTable;
   WORD         wNonResSize;
   FONTDIRENTRY FAR *fdEntry;
   HANDLE       hResource;
   HINSTANCE    hModule;


   fFile = OpenFile(lpszFile, &of, OF_READ );
   if (!fFile)
        return NULL;
   _lread(fFile, (LPSTR)&(OldExe), sizeof(OLDEXE));    
   _llseek(fFile, OldExe.lNewExeOffset, 0 );
   _lread(fFile, (LPSTR)&(NewExe), sizeof(NEWEXE));    
   lNonResTable = NewExe.lNonResOffset;
   _llseek( fFile, lNonResTable, 0 );
   wNonResSize = 0;
   // Read string size
   _lread(fFile, (LPSTR)&wNonResSize, 1);
   _lread(fFile, (LPSTR)lpszDesc, wNonResSize);
   _lclose( fFile );        
   lpszDesc[wNonResSize] = '\0';
   if (lpszDesc[0] == '\0')
      return FALSE; 
   if ((hModule = LoadLibrary(lpszFile)) <= 32)
      return FALSE;

   /* Read Font Directory */
   hResource = FindResource(hModule, "fontdir", RT_FONTDIR);
   if (!hResource)
      return FALSE;
   hResource = LoadResource(hModule, hResource);
   if (fdEntry = (FONTDIRENTRY FAR *)LockResource(hResource))
     {  
     LPSTR lpstr = (LPSTR)fdEntry->dfCharTable;
     LPSTR FaceName;
     while (*lpstr == '\0')
        lpstr++;
     FaceName = lpstr;
     lstrcpy(lf->lfFaceName,FaceName);
     
     lf->lfWeight = fdEntry->dfWeight;
     lf->lfItalic = fdEntry->dfItalic;
     lf->lfUnderline = fdEntry->dfUnderline;
     lf->lfStrikeOut = fdEntry->dfStrikeOut;
     lf->lfCharSet = fdEntry->dfCharSet;
     // convert dfPitchAndFamily to lfPitchAndFamily
     lf->lfPitchAndFamily = (fdEntry->dfPitchAndFamily & 0xF1) + 1;
     UnlockResource(hResource);
     FreeResource(hResource);     
     FreeLibrary(hModule);
     return TRUE;
     }
   else
     return FALSE;  
   }


//****************************************************************************
// Function: ReadLongName
//
// Purpose: Reads the Face Name of a TrueType font.
//
// Parameters:
//    pszFile     == Name of file to read including path
//
// Returns: Face name of the font in the .TTF file
//
// Comments:
//
// History:  Date       Author        Reason
//           3/4/95     GGB           Created
//****************************************************************************

LPSTR ReadLongName(PSTR pszFile)
  {   
  static char lpszLongName[256];
  unsigned            i;
  char		      namebuf[255];
  int                 fp;
  unsigned short      numNames;
  long		      curseek;
  unsigned            cTables;
  sfnt_OffsetTable    OffsetTable;
  sfnt_DirectoryEntry Table;
  sfnt_NamingTable    NamingTable;
  sfnt_NameRecord     NameRecord;

  lpszLongName[0] = '\0';
  if ((fp = open (pszFile, O_RDONLY | O_BINARY)) == -1)
     return NULL;

  /* First off, read the initial directory header on the TTF.  We're only
   * interested in the "numOffsets" variable to tell us how many tables
   * are present in this file.  
   *
   * Remember to always convert from Motorola format (Big Endian to 
   * Little Endian).
   */
  read (fp, &OffsetTable, sizeof (OffsetTable) - sizeof (sfnt_DirectoryEntry));
  cTables = (int) SWAPW (OffsetTable.numOffsets);

  for ( i = 0; i < cTables && i < 40; i++)
  {
    if ((read (fp, &Table, sizeof (Table))) != sizeof(Table)) 
       return NULL;
    if (Table.tag == tag_NamingTable)	/* defined in sfnt_en.h */
    {
	/* Now that we've found the entry for the name table, seek to that
	 * position in the file and read in the initial header for this
	 * particular table.  See "True Type Font Files" for information
	 * on this record layout.
	 */
	lseek (fp, SWAPL (Table.offset), SEEK_SET);
	read (fp, &NamingTable, sizeof (NamingTable));
	numNames = SWAPW(NamingTable.count);
	while (numNames--) 
	{
		read (fp, &NameRecord, sizeof (NameRecord));
		curseek = tell(fp);
		if (SWAPW(NameRecord.platformID) == 1 &&
				SWAPW(NameRecord.nameID) == 4) {
			lseek (fp, SWAPW (NameRecord.offset) + 
				SWAPW(NamingTable.stringOffset) + 
				SWAPL(Table.offset), SEEK_SET);
			read (fp, &namebuf, SWAPW(NameRecord.length));
			namebuf[SWAPW(NameRecord.length)] = '\0';
			lstrcpy(lpszLongName,namebuf);
			lseek (fp, curseek, SEEK_SET);
		}
	}
    close (fp);
    return lpszLongName;
    }
  }
  close (fp);
  return FALSE;
  }


//****************************************************************************
// Function: DrawFontInfo
//
// Purpose: Draws font information about the installed font
//
// Parameters:
//    hdc  == Device context to draw to
//
// Returns: none
//
// Comments:
//
// History:  Date       Author        Reason
//           3/10/95     GGB           Created
//****************************************************************************

void DrawFontInfo(HDC hdc)
{ 
  HFONT hOldFont, hFont;
  HRGN hRgn;
  POINT pt[2];
  
  pt[0].x = 10;
  pt[0].y = 450;
  pt[1].x = 500;
  pt[1].y = 700;
  
  LPtoDP(hdc,pt,2);
  hRgn = CreateRectRgn(pt[0].x,pt[0].y,pt[1].x,pt[1].y);
  SelectClipRgn(hdc,hRgn);
  hOldFont = SelectObject(hdc,ghCurrFont);
  TextOut(hdc,15,455,"AaBbCcDdEeFfGgHhIi",18);
  SetTextColor(hdc,RGB(255,0,0));
  TextOut(hdc,15,535,"JjKkLlMmNnOoPpQqRr",18);
  SetTextColor(hdc,RGB(0,0,255));
  TextOut(hdc,15,615,"SsTtUuVvWwXxYyZz",16);
  SelectObject(hdc,hOldFont);
  SelectClipRgn(hdc,NULL);
  DeleteObject(hRgn);
  
  pt[0].x = 10;
  pt[0].y = 50;
  pt[1].x = 500;
  pt[1].y = 400;
  
  LPtoDP(hdc,pt,2);
  hRgn = CreateRectRgn(pt[0].x,pt[0].y,pt[1].x,pt[1].y);
  SelectClipRgn(hdc,hRgn);
  hFont = CreateFixedWidthFont(32,FW_NORMAL);
  hOldFont = SelectObject(hdc,hFont);
  SetTextColor(hdc,RGB(128,64,64));
  TextOut(hdc,(int)GetTextExtent(hdc,"Font Name:",10)+30,80,
       ghFontInfo.FontName,lstrlen(ghFontInfo.FontName));
  TextOut(hdc,(int)GetTextExtent(hdc,"Font Type:",10)+30,130,
       ghFontInfo.FontType,lstrlen(ghFontInfo.FontType));
  TextOut(hdc,(int)GetTextExtent(hdc,"File:",5)+30,180,
       ghFontInfo.File,lstrlen(ghFontInfo.File));
  TextOut(hdc,(int)GetTextExtent(hdc,"FOT File:",9)+30,230,
       ghFontInfo.FOTFile,lstrlen(ghFontInfo.FOTFile));
  if (ghFontInfo.Scalable)
     TextOut(hdc,(int)GetTextExtent(hdc,"Scalable?",9)+30,280,"Yes",3);
  else
     TextOut(hdc,(int)GetTextExtent(hdc,"Scalable?",9)+30,280,"No",2);
  TextOut(hdc,(int)GetTextExtent(hdc,"Pitch:",6)+30,330,ghFontInfo.Pitch,lstrlen(ghFontInfo.Pitch));
  DeleteObject(SelectObject(hdc,hOldFont));
  SelectClipRgn(hdc,NULL);
  DeleteObject(hRgn);
  DrawMetricInfo(hdc);
}


//****************************************************************************
// Function: DrawMetricInfo
//
// Purpose: Draws TextMetric information about the installed font
//
// Parameters:
//    hdc  == Device context to draw to
//
// Returns: none
//
// Comments:
//
// History:  Date       Author        Reason
//           3/10/95     GGB           Created
//****************************************************************************

void DrawMetricInfo(HDC hdc)
{  
  TEXTMETRIC tm;
  HFONT hOldFont, hFont;
  HRGN hRgn;
  POINT pt[2];
  int y  = 21;

  pt[0].x = 510;
  pt[0].y = 50;
  pt[1].x = 990;
  pt[1].y = 700;
  
  LPtoDP(hdc,pt,2);
  hRgn = CreateRectRgn(pt[0].x,pt[0].y,pt[1].x,pt[1].y);
  SelectClipRgn(hdc,hRgn);
  hOldFont = SelectObject(hdc,ghCurrFont);
  GetTextMetrics(hdc,&tm);
  SelectObject(hdc,hOldFont);
  hFont = CreateFixedWidthFont(29,FW_NORMAL);
  hOldFont = SelectObject(hdc,hFont);
  SetTextColor(hdc,RGB(128,64,64));
  DrawExtentLine(hdc, "tmHeight:", y+=32, tm.tmHeight);
  DrawExtentLine(hdc, "tmAscent:", y+=32, tm.tmAscent);
  DrawExtentLine(hdc, "tmDescent:", y+=32, tm.tmDescent);
  DrawExtentLine(hdc, "tmInternalLeading:", y+=32, tm.tmInternalLeading);
  DrawExtentLine(hdc, "tmExternalLeading:", y+=32, tm.tmExternalLeading);
  DrawExtentLine(hdc, "tmAveCharWidth:", y+=32, tm.tmAveCharWidth);
  DrawExtentLine(hdc, "tmMaxCharWidth:", y+=32, tm.tmMaxCharWidth);
  DrawExtentLine(hdc, "tmWeight:", y+=32, tm.tmWeight);
  DrawExtentLine(hdc, "tmItalic:", y+=32, tm.tmItalic);
  DrawExtentLine(hdc, "tmUnderlined:", y+=32, tm.tmUnderlined);
  DrawExtentLine(hdc, "tmStruckOut:", y+=32, tm.tmStruckOut);
  DrawExtentLine(hdc, "tmFirstChar:", y+=32, tm.tmFirstChar);
  DrawExtentLine(hdc, "tmLastChar:", y+=32, tm.tmLastChar);
  DrawExtentLine(hdc, "tmDefaultChar:", y+=32, tm.tmDefaultChar);
  DrawExtentLine(hdc, "tmBreakChar:", y+=32, tm.tmBreakChar);
  DrawExtentLine(hdc, "tmPitchAndFamily:", y+=32, tm.tmPitchAndFamily);
  DrawExtentLine(hdc, "tmCharSet:", y+=32, tm.tmCharSet);
  DrawExtentLine(hdc, "tmOverhang:", y+=32, tm.tmOverhang);
  DrawExtentLine(hdc, "tmDigitizedAspectX:", y+=32, tm.tmDigitizedAspectX);
  DrawExtentLine(hdc, "tmDigitizedAspectY:", y+=32, tm.tmDigitizedAspectY);
 
  DeleteObject(SelectObject(hdc,hOldFont));
  SelectClipRgn(hdc,NULL);
  DeleteObject(hRgn);
}


//****************************************************************************
// Function: DrawExtentLine
//
// Purpose: Draws a line of TextMetric information
//
// Parameters:
//    hdc         == Device context to draw to
//    lpszString  == String to draw before actual value
//    nMetric     == value to draw after string
//
// Returns: none
//
// Comments:
//
// History:  Date       Author        Reason
//           3/10/95     GGB           Created
//****************************************************************************

void DrawExtentLine(HDC hdc, LPSTR lpszString, int y, int nMetric)
{
  char szBuf[80];
  
  wsprintf(szBuf,"%d",nMetric);
  TextOut(hdc, (int)GetTextExtent(hdc,lpszString,lstrlen(lpszString))+530, 
          y, szBuf, lstrlen(szBuf));
}
